<?php

namespace Drupal\group_member_mgmt;

use Drupal\Core\Session\AccountInterface;
use Drupal\group\Access\CalculatedGroupPermissionsItemInterface;
use Drupal\group\Access\ChainGroupPermissionCalculatorInterface;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Entity\GroupTypeInterface;

/**
 * Class GroupMemberAccessHandler.
 */
class GroupMemberAccessHandler {

  /**
   * The group permission calculator.
   *
   * @var \Drupal\group\Access\ChainGroupPermissionCalculatorInterface
   */
  protected $groupPermissionCalculator;

  /**
   * GroupMemberAccessHandler constructor.
   *
   * @param \Drupal\group\Access\ChainGroupPermissionCalculatorInterface $permission_calculator
   *   The group permission calculator.
   */
  public function __construct(ChainGroupPermissionCalculatorInterface $permission_calculator) {
    $this->groupPermissionCalculator = $permission_calculator;
  }

  /**
   * Gets group membership permissions for a group type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface|null $groupType
   *   The group type, may be null in some cases.
   * @param array $permissions
   *   An array of group permissions to add on to. Optional, defaults to an
   *   empty set of permissions.
   *
   * @return array
   *   Array of group permission definitions.
   */
  public function getGroupMembershipPermissions($groupType, array $permissions = []) {
    if (!$groupType) {
      return $permissions;
    }

    // Adjust title on existing "create" permission.
    if (isset($permissions['create group_membership entity'])) {
      $permissions['create group_membership entity']['title'] = 'Entity: Add any %entity_type entities';
    }

    // Add/edit/delete permissions for members with each role.
    foreach ($this->getGroupMemberRoles($groupType) as $role) {
      $args = ['%role' => $role->label()];
      $permissions['group_member_mgmt add group_membership role ' . $role->id()] = [
        'title' => 'Member: Add members with group role %role',
        'title_args' => $args,
        'description' => 'Allows you to add a new group member with the role %role.',
        'description_args' => $args,
      ];

      $permissions['group_member_mgmt edit group_membership role ' . $role->id()] = [
        'title' => 'Member: Edit members with group role %role',
        'title_args' => $args,
        'description' => 'Allows you to edit a new group member with the role %role.',
        'description_args' => $args,
      ];

      $permissions['group_member_mgmt delete group_membership role ' . $role->id()] = [
        'title' => 'Member: Delete members with group role %role',
        'title_args' => $args,
        'description' => 'Allows you to delete group members with the role %role.',
        'description_args' => $args,
      ];
    }

    return $permissions;
  }

  /**
   * Checks access for creating members.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to check for content creation access.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user for which to check access.
   *
   * @return bool
   *   Indicates if the account has access.
   */
  public function createMemberAccess(GroupInterface $group, AccountInterface $account) {
    $item = $this->groupPermissionCalculator
      ->calculatePermissions($account)
      ->getItem(CalculatedGroupPermissionsItemInterface::SCOPE_GROUP, $group->id());

    // Any type of "add" permission will allow access.
    $permissions = array_filter($item->getPermissions(), function ($permission) {
      return preg_match('/^group_member_mgmt add group_membership/', $permission);
    });

    return count($permissions) > 0;
  }

  /**
   * Checks access for updating members.
   *
   * @param \Drupal\group\Entity\GroupContentInterface $group_content
   *   The group content entity.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user for which to check access.
   *
   * @return bool
   *   Indicates if the account has access.
   */
  public function updateMemberAccess(GroupContentInterface $group_content, AccountInterface $account) {
    foreach ($group_content->get('group_roles')->referencedEntities() as $groupRole) {
      /** @var \Drupal\group\Entity\GroupRoleInterface $groupRole */
      if ($group_content->getGroup()->hasPermission('group_member_mgmt edit group_membership role ' . $groupRole->id(), $account)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Checks access for deleting members.
   *
   * @param \Drupal\group\Entity\GroupContentInterface $group_content
   *   The group content entity.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user for which to check access.
   *
   * @return bool
   *   Indicates if the account has access.
   */
  public function deleteMemberAccess(GroupContentInterface $group_content, AccountInterface $account) {
    foreach ($group_content->get('group_roles')->referencedEntities() as $groupRole) {
      /** @var \Drupal\group\Entity\GroupRoleInterface $groupRole */
      if ($group_content->getGroup()->hasPermission('group_member_mgmt delete group_membership role ' . $groupRole->id(), $account)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Gets roles for members of the group.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The group type.
   *
   * @return array|\Drupal\group\Entity\GroupRoleInterface[]
   *   Group roles.
   */
  protected function getGroupMemberRoles(GroupTypeInterface $groupType) {
    $roles = $groupType->getRoles(TRUE);
    $roles = array_filter($roles, function ($role) {
      /** @var \Drupal\user\RoleInterface $role */
      return $role->get('audience') == 'member';
    });
    return $roles;
  }

}
